;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Atari Jaguar JETPAC Conversion
;;
;; Original System:    Sinclair ZX-Spectrum 16k
;; Original Copyright: Ultimate Play the Game (RARE)
;;
;; Code by Cyrano Jones
;;

; Notes:
;
; Spectrum version has: Maximum of 6 nasties
;                       Bonus items fall after 1st stage assembled
;                       Fuel after all stages assembled
;
;                       Holding top row keys causes a 'hover'
;
                
                .include    "jaguar.inc"
                .text

                .extern readpad
                .extern handle_input
                .extern turn_left
                .extern turn_right
                .extern go_up
                .extern go_down
                .extern fire
                .extern fire2
                .extern go_up
                .extern go_down

objs            equ     30
ob_list         equ     $100000+(850*1024)      ; the object list in use
ob_list1        equ     $100000+(860*1024)      ; the shadow object list

top_of_the_code:

                move.w  #$2700,sr               ; kill interupts

                lea     $1ffff8,a7              ; set up stack

                move.l  #0,INT1                 ; CPU interupt CTRL reg
                move.w  #-1,VI                  ; Disable Vertical Interupts
                move.w  #$4c7,VMODE             ; 16 bit RGB mode

                moveq.l #0,d0
                move.l  d0,L_I2S                ; clear serial interface ints
                move.l  d0,R_I2S
                move.l  d0,G_CTRL               ; clear GPU ctrl
                move.l  d0,D_CTRL               ; clear DSP ctrl
                move.l  d0,G_FLAGS              ;
                move.l  d0,D_FLAGS              ;
                move.l  #$70007,G_END           ; big-endian mode
                move.l  #$70007,D_END

                lea     $4000,a0
                lea     $b000+(objs*24),a1
                jsr     clear_memory

                jsr     InitVideo               ; Setup our video registers.
                jsr     Init_music
                bsr     ConvertBMPtoJAG
                bsr     MakeObjectList         ; Create bit magled list

                lea     $f00400,a0
                move.l  #255,d7
.blackpal:      clr.w   (a0)+
                dbf     d7,.blackpal

                move.l  #VB_INTERUPT,LEVEL0    ; Install 68K LEVEL0 handler
                move.w  a_vde,d0                ; Must be ODD
                sub.w   #16,d0
                ori.w   #1,d0
                move.w  d0,VI

                move.w  #1,INT1                 ; Enable video interrupts
                and.w   #$f8ff,sr

                bsr     wait_frame             ; Stop the OP from freaking

                move.l  #ob_list,d0
                swap    d0
                move.l  d0,OLP                  ; Object Processor is GO!

gameloop:       bsr     title_sequence
                bsr     game_code

                bra     gameloop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Title Sequence Code
;;
;;

title_sequence: move.l  screen_obj,a0
                move.l  title_addr,a1                       
                move.l  #bitmapobject,d0        ; type
                move.l  #230,d1                 ; height
                move.l  #448,d2                 ; bytes to next line
                move.l  #3,d3                   ; 8-Bit
                move.l  #0,d4                   ; non transparent
                move.l  #448,d5                 ; bytes/pixels*width / line
                jsr     CreateObject
                
                move.l  screen_obj,a0
                and.w   #$f000,14(a0)
                and.w   #$c007,6(a0)
                or.w    #364*8,6(a0)
                move.l  a0,scrmove_obj

                lea     title_pal,a1
                bsr     fadepal_up

.loop:          jsr     readpad
                jsr     handle_input

                tst.w   fire
                beq.s   .loop

                bsr     fadepal_down

                rts

jetman_x:       dc.w    0
jetman_y:       dc.w    0

lives:          dc.w    0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; JETPAC Game Code
;;
;;

game_code:      move.l  screen_obj,a0
                move.l  gamescr_addr,a1                       
                move.l  #bitmapobject,d0        ; type
                move.l  #230,d1                 ; height
                move.l  #448,d2                 ; bytes to next line
                move.l  #3,d3                   ; 8-Bit
                move.l  #0,d4                   ; non transparent
                move.l  #448,d5                 ; bytes/pixels*width / line
                jsr     CreateObject
                
                clr.w   rocket_built
                clr.w   mid_in_place
                move.w  #0,fuel_filled
                move.w  #2,lives
                move.b  #'3',jetmen
                jsr     print_lives

                move.l  screen_obj,a0
                and.w   #$f000,14(a0)
                and.w   #$c007,6(a0)
                or.w    #364*8,6(a0)
                move.l  a0,scrmove_obj          ; screen start position

                lea     game_pal,a1
                bsr     fadepal_up              ; fade'n'move the screen
                
                lea     $f00400+(249*2),a0
                moveq   #5,d7
.whiteout:      move.w  #-1,(a0)+
                dbf     d7,.whiteout
                
                bsr     setup_initial_objects   ; give the objects x/y pos
                bsr     init_monsters

                move.w  #1,inside_game

.loop:          clr.w   lr_pressed              ; left/right NOT pressed

                jsr     readpad                 ; read the uncomfortable one
                jsr     handle_input            ; and process the pain

.check_left:    tst.w   turn_left
                beq.s   .check_right
                bsr     move_jetman_left        ; Joypad LEFT pressed
.check_right:   tst.w   turn_right
                beq.s   .check_up
                bsr     move_jetman_right       ; Joypad RIGHT pressed
.check_up:      tst.w   go_up
                beq.s   .nostick
                bsr     move_jetman_up          ; Joypad UP pressed
                bra.s   .skipgravity

.nostick:       bsr     jetman_gravity          ; Gravity (on the moon!?!)

.skipgravity:   bsr     process_rocket          ; construction work?
                bsr     process_fuel            ; fill 'er up!
                bsr     leaving_planet          ; let's get outta town....?

                bsr     plot_jetman             ; Stick 'im on screen at x/y
                bsr     plot_monsters           ; draw all the moving ones...     
                
                bsr     check_jetman_hit_monster
                tst.w   d7                      ; is our hero dead?
                beq.s   .normal
                sub.w   #1,lives
                sub.b   #1,jetmen
                
                bsr     init_monsters           ; reset the loonies!
                bsr     init_jetman             ; reset Jetdude!

.normal:        tst.w   fire
                beq.s   .no_fire
                bsr     new_laser               ; add to laser list
.no_fire:       
                bsr     wait_frame              ; video sync

                tst.w   lives
                bpl     .loop                   ; game over man, game over.

                move.l  jetman_obj,a0
                bsr     hide_object
                move.l  rocketa_obj,a0
                bsr     hide_object
                move.l  rocketb_obj,a0
                bsr     hide_object
                move.l  rocketc_obj,a0
                bsr     hide_object
                move.l  fuel_obj,a0                
                bsr     hide_object

                lea     monster1_obj,a1
                moveq   #5,d7
.killmnst:      move.l  (a1)+,a0
                bsr     hide_object
                dbf     d7,.killmnst

                move.w  #60,d7
.hangon:        bsr     wait_frame
                dbf     d7,.hangon

                clr.w   inside_game

                bsr     fadepal_down

                rts

hide_object:    and.w   #$c007,6(a0)
                move.w  #-100*8,d0
                and.w   #$3ff8,d0
                or.w    d0,6(a0)
                rts    

lr_pressed:     dc.w    0
inside_game:    dc.w    0


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Dodgy monster collision detection
;;
;;

check_jetman_hit_monster:
                lea     monster_table,a1        ; monster data table
                moveq   #5,d6
.loop:          move.l  a1,-(a7)
                move.l  d6,-(a7)
                bsr     .monster_check          ; check if hit
                move.l  (a7)+,d6
                move.l  (a7)+,a1
                tst.w   d7
                bne.s   .out
                lea     8(a1),a1                ; next monster
                dbf     d6,.loop
.out:           rts
                
.monster_check: moveq   #0,d7

                move.w  (a1),d0                 ; x-pos
                move.w  2(a1),d1                ; y-pos

                move.w  jetman_x,d2             ; jetman x-pos
                move.w  jetman_y,d3             ; jetman y-pos
                
                tst.w   d2
                bpl.s   .poscheck

.negcheck:      add.w   #100,d2      
                add.w   #100,d0

.poscheck:      move.w  d2,d4                   ; check if on same x-values
                add.w   #8,d4
                sub.w   d0,d4                   ;
                bmi.s   .fail
                cmp.w   #16,d4
                bge.s   .fail

                move.w  d3,d4                   ; check if on same y-values
                add.w   #16,d4
                sub.w   d1,d4
                bmi.s   .fail
                cmp.w   #24,d4
                bge.s   .fail

                moveq   #1,d7                   ; hit that sprite!
.fail:          rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Monster Routines
;;
;;

init_monsters:  lea     monster_table,a0
                moveq   #5,d7                   ; 6 monsters
.make:          move.w  #490,(a0)+              ; init x-pos
                move.w  random_number,d0
                divu    #180,d0
                swap    d0
                add.w   #50,d0
                move.w  d0,(a0)+                ; init y-pos
                and.w   #3,d0
                add.w   #2,d0
                move.w  d0,(a0)+                ; init x-velocity
                swap    d0
                and.w   #2,d0
                move.w  d0,(a0)+                ; init y-velocity
                bsr     wait_frame
                dbf     d7,.make
                rts

monster_type:   dc.w    0

plot_monsters:  lea     monster_table,a4
                lea     monster1_obj,a5

                move.w  monster_type,d7
                add.w   d7,d7
                add.w   d7,d7
                move.l  .monster_logic(pc,d7.w),a6
                jmp     (a6)                    ; relevant monsters


.monster_logic: dc.l    ASTEROIDS

ASTEROIDS:      clr.w   .do_drop
                add.w   #1,.asteroid_drop
                cmp.w   #3,.asteroid_drop
                bne.s   .proc_roids
                clr.w   .asteroid_drop
                move.w  #1,.do_drop
.proc_roids:

                moveq   #5,d7
.plloop:        move.w  (a4),d0                 ; x
                move.w  2(a4),d1                ; y
                move.w  4(a4),d2                ; x-vel
                move.w  6(a4),d3                ; y-vel
                sub.w   d2,d0                   ; make new x-pos
                
                tst.w   .do_drop
                beq.s   .nodown
                add.w   d3,d1                   ; make new y-pos
.nodown:        
                movem.l d0-d7/a0-a6,-(a7)
                move.w  #0,.hit_obj
                move.l  gamescr_addr,a0
                sub.w   #20,d1
                mulu    #448,d1
                add.l   d1,a0
                add.w   d0,a0
                moveq   #13,d7
.down:          moveq   #15,d6
.over:          tst.b   (a0)+                   ; check monster not hitting
                bne.s   .failed                 ; any bullets
                dbf     d6,.over
                lea     448-16(a0),a0
                dbf     d7,.down
                move.w  #1,.hit_obj
.failed:        movem.l (a7)+,d0-d7/a0-a6

                tst.w   .hit_obj
                bne.s   .nohit

.makenewobj:    bsr     .recreateobj            ; make new object co-ords
                move.w  (a4),d0                 ; x
                move.w  2(a4),d1                ; y
                bra.s   .final

.nohit:         tst.w   d0                      ; check if off left
                bmi.s   .makenewobj
                cmp.w   #493,d0
                bge.s   .makenewobj

                move.w  d0,(a4)
                move.w  d1,2(a4)

.final:         move.l  (a5)+,a0                ; address of object
                move.w  4(a4),d2
                bsr     update_monster
                lea     8(a4),a4
                dbf     d7,.plloop
                rts

.recreateobj:   
                move.w  #490,(a4)               ; init x-pos
                move.w  random_number,d0
                divu    #180,d0
                swap    d0
                add.w   #50,d0
                move.w  d0,2(a4)                ; init y-pos
                and.w   #3,d0
                add.w   #2,d0
                move.w  d0,4(a4)                ; init x-velocity
                swap    d0
                and.w   #2,d0
                move.w  d0,6(a4)                ; init y-velocity
                
                tst.w   .asteroid_drop
                bne.s   .make_l_2_r
                rts
.make_l_2_r:    clr.w   (a4)
                neg.w   4(a4)
                rts

.hit_obj:       dc.w    0
.asteroid_drop: dc.w    0
.do_drop:       dc.w    0

update_monster: 
                movem.l d0-d7/a0-a6,-(a7)
                move.w  monster_type,d5
                lsl.w   #3,d5
                lea     monster_animations,a3
                move.l  (a3,d5.w),a1            ; pointer to right anim ptr
                move.l  4(a3,d5.w),a2           ; pointer to left anim ptr

                move.l  (a1),a3                 ; pointer into r-table
                move.l  (a2),a4                 ; pointer into l-table

                move.l  (a3),d5
                lsl.l   #8,d5
                and.l   #$fffff800,d5           ; next animation frame

                move.l  (a4),d4
                lsl.l   #8,d4
                and.l   #$fffff800,d4           ; next animation frame

                tst.w   d2                      ; direction check!
                bpl.s   .right
                move.l  d4,d6
                bra.s   .saveit
.right:         move.l  d5,d6

.saveit:        move.l  (a0),d5
                and.l   #$000007ff,d5
                or.l    d6,d5
                move.l  d5,(a0)

                move.w  6(a0),d6
                and.w   #$c007,d6
                add.w   d1,d1
                lsl.w   #3,d1
                or.w    d1,d6
                move.w  d6,6(a0)
                
                move.w  14(a0),d6
                and.w   #$f000,d6
                and.w   #$fff,d0
                or.w    d0,d6
                move.w  d6,14(a0)

                movem.l (a7)+,d0-d7/a0-a6
                rts

monster_table:  dc.w    0,0,0,0                 ; x,y,x-vel,y-vel
                dc.w    0,0,0,0
                dc.w    0,0,0,0
                dc.w    0,0,0,0
                dc.w    0,0,0,0
                dc.w    0,0,0,0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Add new laser to the list
;;
;;

new_laser:      lea     laser_table,a0          ; laser table
                moveq   #3,d7                   ; maximum of 4 lasers
.find_pos:      cmp.w   #-1,6(a0)               ; check if slot free
                beq.s   .save_here
                lea     12(a0),a0               ; next slot
                dbf     d7,.find_pos            ; fall out if all full
                rts

.save_here:     move.w  jetman_x,d0             ; get x-pos of jetman
                tst.w   facing
                bne.s   .left
.right:         add.w   #16,d0
                move.w  #16,4(a0)               ; right additive
                bra.s   .saveit
.left:          move.w  #-16,4(a0)              ; left additive
.saveit:        move.w  d0,(a0)                 ; initial x
                move.w  jetman_y,d0             ; initial y
                sub.w   #6,d0                   ; offset to gun!
                move.w  d0,2(a0)                ; initial y
                move.w  #20,6(a0)               ; framecounter
                move.l  #empty,8(a0)            ; 1st pixel killed here
                rts

laser_table:    dc.w    -1,-1,-1,-1,0,0                
                dc.w    -1,-1,-1,-1,0,0
                dc.w    -1,-1,-1,-1,0,0
                dc.w    -1,-1,-1,-1,0,0
empty:          dc.l    0

; start-x/start-y/direction/frame/curent-x/last bullet

plot_lasers:    lea     laser_table,a0
                moveq   #3,d7                   ; 4 lasers max
.laserloop:     cmp.w   #-1,6(a0)               ; check frame counter     
                beq.s   .notfiring
                bsr     .active_laser     
.notfiring:     lea     12(a0),a0
                dbf     d7,.laserloop
                rts

.active_laser:  move.l  8(a0),a3
                moveq   #0,d0
                moveq   #0,d1
                move.l  gamescr_addr,a1
                move.w  (a0),d0                 ; x
                move.w  2(a0),d1                ; y
                mulu    #448,d1
                add.l   d0,d1
                add.l   d1,a1                   ; address of start point
        
                sub.w   #1,6(a0)                ; sub frame counter
                beq.s   .erase_final            ; if end then kill last pixel

                add.w   4(a0),a1                ; add the velocity on
                tst.b   (a1)                    ; test a byte at current pos
                beq     .ok_2_plot              ; if 0 then plot fire
                move.w  #1,6(a0)                ; tell it last frame
                rts
.ok_2_plot:     move.b  #200,(a1)               ; plot the bullet
                move.l  a1,8(a0)                ; store address on screen
                clr.b   (a3)                    ; clear last bullet
                add.w   4(a0),d0                ; add velocity to x-pos
                
                bpl.s   .ok                     ; if +ve then no l-wrap
                add.w   #448,d0                 ; corrective...
.ok:
                
                move.w  d0,(a0)                 ; store new x-pos
                rts

.erase_final:   clr.b   (a3)
                move.l  #$ffffffff,(a0)
                move.l  #$ffffffff,4(a0)
                move.l  #$ffffffff,8(a0)
                rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Leaving Planet Routine         
;;
;;

leaving_planet:
                tst.w   rocket_built
                bne.s   .leaving
.rts:           rts

.leaving:       
                lea     rocketcx,a1
                
                move.w  (a1),d0                 ; x-pos
                move.w  2(a1),d1                ; y-pos
                move.w  jetman_x,d2             ; jetman x-pos
                move.w  jetman_y,d3             ; jetman y-pos
                move.w  d2,d4                   ; check if on same x-values
                add.w   #8,d4                   ;
                sub.w   d0,d4                   ;
                bmi     .fail
                cmp.w   #16,d4
                bge     .fail
                move.w  d3,d4                   ; check if on same y-values
                add.w   #16,d4
                sub.w   d1,d4
                bmi     .fail
                cmp.w   #22,d4
                bge     .fail

                cmp.w   #6,fuel_filled       
                bne     .fail

                move.w  #1,flash_rocket

                lea     $f00400+(250*2),a0
                move.w  #5,d0
.white:         move.w  #$ffff,(a0)+            ; make all rocket white
                dbf     d0,.white

                move.l  fuel_obj,a0
                bsr     hide_object

                move.l  jetman_obj,a0
                bsr     hide_object

                move.w  #0,fuel_wait_time
                move.w  #0,fuel_filled
                move.w  #1,new_fuel
                move.w  #260,jetman_x           ; setup jetman
                move.w  #212,jetman_y
                clr.w   ranim_offset
                clr.w   lanim_offset
                clr.w   facing
                clr.w   flash_rocket

                move.w  #4,.flight_dir
                moveq   #1,d4
.down:          moveq   #38,d5
.up:            move.w  .flight_dir,d0
                sub.w   d0,rocketcy
                sub.w   d0,rocketby
                sub.w   d0,rocketay
                move.l  rocketc_obj,a0
                move.w  rocketcx,d0
                move.w  rocketcy,d1
                bsr     update_rocket
                move.l  rocketb_obj,a0
                move.w  rocketbx,d0
                move.w  rocketby,d1
                bsr     update_rocket
                move.l  rocketa_obj,a0
                move.w  rocketax,d0
                move.w  rocketay,d1
                movem.l d0-d7/a0-a6,-(a7)       ; keep
                bsr     update_rocket
                bsr     plot_monsters           ;  em
                movem.l (a7)+,d0-d7/a0-a6       ;   moving!
                bsr     wait_frame
                dbf     d5,.up
                cmp.w   #2,.launches
                bne.s   .stillok
                clr.w   .launches
                bra     setup_initial_objects
.stillok:       neg.w   .flight_dir
                movem.l d0-d7/a0-a6,-(a7)
                bsr     init_monsters
                movem.l (a7)+,d0-d7/a0-a6
                
                dbf     d4,.down
                add.w   #1,.launches                

.fail:          rts

.flight_dir:    dc.w    0
.launches:      dc.w    0


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Process Fuel Routine           
;;
;;

process_fuel:   tst.w   rocket_built
                bne.s   .drop_fuel
.rts:           rts

.drop_fuel:     tst.w   new_fuel
                beq.s   .old_fuel
                sub.w   #1,fuel_wait_time
                bpl.s   .rts
                bra     .create_new_fuel

.old_fuel:      moveq   #0,d7
                lea     fuel_x,a1
                move.l  fuel_obj,a0
                bsr     .check_fuel_over_rocket         ; ok
                tst.w   d7
                bne     .drop_fuel_onto_rocket          ; ok
                bsr     .check_fuel_under_jetman        ; ok
                tst.w   d7
                bne     .move_fuel_with_jetman          ; ok
                bsr     .check_fuel_dropping            ; ok
                tst.w   d7
                bne     .fuel_dropping
.new:           rts

.create_new_fuel:
                clr.w   new_fuel
                move.w  #20,fuel_y
                move.w  random_number,d0
                move.w  d0,d1
                and.w   #$fff0,d0
                move.w  d0,fuel_x
                and.w   #$f,d1
                lsl.w   #5,d1
                move.w  d1,fuel_wait_time
                rts
.check_fuel_over_rocket:
                moveq   #0,d7                   ; check if at x-pos 290
                cmp.w   #290,(a1)               ; (the rocket)
                bne.s   .rts0
                moveq   #1,d7
.rts0:          rts

.drop_fuel_onto_rocket:
                move.w  2(a1),d1                ; y-pos
                move.w  4(a1),d2
                cmp.w   d1,d2
                bgt     .fuel_dropping
                move.l  fuel_obj,a0
                bsr     hide_object
                move.w  #-20,fuel_x
                move.w  #10,fuel_y
                add.w   #1,fuel_filled
                cmp.w   #6,fuel_filled
                beq.s   .rts1
                move.w  #1,new_fuel
.rts1:          rts

.check_fuel_under_jetman:
                clr.w   carry_fuel
                moveq   #0,d7
                move.w  (a1),d0                 ; x-pos
                move.w  2(a1),d1                ; y-pos

                move.w  jetman_x,d2             ; jetman x-pos
                move.w  jetman_y,d3             ; jetman y-pos
                
                tst.w   d2
                bpl.s   .poscheck

.negcheck:      add.w   #100,d2      
                add.w   #100,d0

.poscheck:      move.w  d2,d4                   ; check if on same x-values
                add.w   #8,d4                   ;
                sub.w   d0,d4                   ;
                bmi.s   .fail
                cmp.w   #16,d4
                bge.s   .fail

                move.w  d3,d4                   ; check if on same y-values
                add.w   #16,d4
                sub.w   d1,d4
                bmi.s   .fail
                cmp.w   #22,d4
                bge.s   .fail

                move.w  #1,carry_fuel
                moveq   #1,d7                   ; hit that sprite!
.fail:          rts

.move_fuel_with_jetman:
                move.w  jetman_x,d0
                move.w  jetman_y,d1
                add.w   #8,d1
                move.w  d0,(a1)
                move.w  d1,2(a1)
                bsr     update_fuel
                rts

.check_fuel_dropping:
                moveq   #0,d7
                move.w  4(a1),d6
                move.w  fuel_y,d5
                cmp.w   d6,d5                   ; bottom of screen?
                bge.s   .nodrop
                moveq   #0,d0
                move.w  fuel_y,d0              ; get y-value
                sub.w   #5,d0                   ; get bottom of jetman
                mulu    #448,d0
                add.w   fuel_x,d0
                move.l  gamescr_addr,a3
                add.l   d0,a3                   ; offset into screen
                moveq   #15,d6                  ; 16 pixels to test
.testloop:      tst.b   (a3)+                   ; check 
                bne.s   .nodrop                 ;  below (& branch if hit!)
                dbf     d6,.testloop            ;   jetman
                moveq   #1,d7                   ; down!
.nodrop:        rts

.fuel_dropping: move.w  2(a1),d1
                cmp.w   4(a1),d1
                bgt.s   .no_more
                add.w   #1,d1
.no_more:       move.w  d1,2(a1)
                move.w  (a1),d0
                bsr     update_fuel
                rts

fuel_x:         dc.w    0
fuel_y:         dc.w    0
fuel_final:     dc.w    225
fuel_wait_time: dc.w    0


update_fuel:    move.w  6(a0),d6
                and.w   #$c007,d6
                add.w   d1,d1
                lsl.w   #3,d1
                or.w    d1,d6
                move.w  d6,6(a0)
                
                move.w  14(a0),d6
                and.w   #$f000,d6
                and.w   #$fff,d0
                or.w    d0,d6
                move.w  d6,14(a0)
                rts

new_fuel:       dc.w    0


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Process Rocket Routine         
;;
;;

rocket_built:   dc.w    0
mid_in_place:   dc.w    0
fuel_filled:    dc.w    0

process_rocket: tst.w   rocket_built
                beq.s   .nope                   ; work to do!
                rts                             ; all done!

.nope:          tst.w   mid_in_place            ; test if middle there!                    
                beq     .handle_middle          ; nope!

.handle_top:    move.l  rocketa_obj,a0
                lea     rocketax,a1
                bra     .shift_rocket

.handle_middle: move.l  rocketb_obj,a0
                lea     rocketbx,a1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;    a0 pointing to rocket segment object to process
;;  (a1) rocket x-position
;; 4(a1) rocket y-position
;; 6(a1) rocket final y-pos

.shift_rocket:  bsr     .check_over_rocket_bottom
                tst.w   d7
                bne     .drop_rocket_onto_base
                bsr     .check_under_jetman
                tst.w   d7
                bne     .move_with_jetman
                bsr     .check_dropping
                tst.w   d7
                bne     .drop_rocket
                rts

.check_over_rocket_bottom:
                moveq   #0,d7                   ; check if at x-pos 290
                cmp.w   #290,(a1)               ; (the rocket)
                bne.s   .rts0
                moveq   #1,d7
.rts0:          rts

.check_under_jetman:
                clr.w   carry_rocket
                
                moveq   #0,d7
                move.w  (a1),d0                 ; x-pos
                move.w  2(a1),d1                ; y-pos

                move.w  jetman_x,d2             ; jetman x-pos
                move.w  jetman_y,d3             ; jetman y-pos
                
                tst.w   d2
                bpl.s   .poscheck

.netcheck:      add.w   #100,d2
                add.w   #100,d0

.poscheck:      move.w  d2,d4                   ; check if on same x-values
                add.w   #8,d4                   ;
                sub.w   d0,d4                   ;
                bmi.s   .fail
                cmp.w   #16,d4
                bge.s   .fail

                move.w  d3,d4                   ; check if on same y-values
                add.w   #16,d4
                sub.w   d1,d4
                bmi.s   .fail
                cmp.w   #22,d4
                bge.s   .fail

                move.w  #1,carry_rocket
                move.l  a1,rocket_pointer
                moveq   #1,d7                   ; hit that sprite!
.fail:          rts

.check_dropping:
                moveq   #0,d7
                moveq   #0,d0
                move.w  2(a1),d0                ; get y-value
                mulu    #448,d0
                add.w   (a1),d0
                move.l  gamescr_addr,a3
                add.l   d0,a3                   ; offset into screen
                moveq   #15,d6                  ; 16 pixels to test
.testloop:      tst.b   (a3)+                   ; check 
                bne.s   .nodrop                 ;  below (& branch if hit!)
                dbf     d6,.testloop            ;   rocket piece
                moveq   #1,d7                   ; down!
.nodrop:        rts

                moveq   #0,d7
                rts

.drop_rocket_onto_base:
                move.w  2(a1),d1                ; y-pos
                move.w  4(a1),d2
                cmp.w   d1,d2
                bge.s   .drop_rocket
                move.w  d2,2(a1)
                move.w  (a1),d0
                move.w  d2,d1
                bsr     update_rocket
                
                tst.w   mid_in_place
                bne.s   .all_done
                move.w  #1,mid_in_place
                rts
.all_done:      move.w  #1,rocket_built
                move.w  #1,new_fuel
                rts

.drop_rocket:   move.w  2(a1),d1
                cmp.w   #221,d1
                bgt.s   .no_more
                add.w   #2,d1
.no_more:       move.w  d1,2(a1)
                move.w  (a1),d0
                bsr     update_rocket
                rts

.move_with_jetman:
                move.w  jetman_x,d0
                move.w  jetman_y,d1
                add.w   #8,d1
                move.w  d0,(a1)
                move.w  d1,2(a1)
                bsr     update_rocket
                rts

update_rocket:  move.w  6(a0),d6
                and.w   #$c007,d6
                add.w   d1,d1
                lsl.w   #3,d1
                or.w    d1,d6
                move.w  d6,6(a0)
                
                move.w  14(a0),d6
                and.w   #$f000,d6
                and.w   #$fff,d0
                or.w    d0,d6
                move.w  d6,14(a0)
                rts

carry_rocket:   dc.w    0
carry_fuel:     dc.w    0
rocket_pointer: dc.l    0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Update JETMAN x/y co-ordinates
;;
;;

next_jetman_gfx:        dc.l    0

plot_jetman:    move.l  jetman_obj,a0           ; store build address
                move.l  next_jetman_gfx,a1      ; gfx address
                move.l  #bitmapobject,d0        ; type
                move.l  #24,d1                  ; height
                move.l  #16*2,d2                ; bytes to next line
                move.l  #4,d3                   ; 16bit
                move.l  #1,d4                   ; non transparent
                move.l  #16*2,d5                ; bytes/pixels*width / line
                jsr     CreateObject

                move.w  jetman_x,d0
                and.w   #$fff,d0
                move.w  jetman_y,d1
                add.w   d1,d1                   ; convert to OP y-values
                move.l  jetman_obj,a0
                and.w   #$f000,14(a0)
                or.w    d0,14(a0)
                and.w   #$c007,6(a0)
                lsl.w   #3,d1                   ; mult by 8
                and.w   #$3ff8,d1               ; keep middle bits
                or.w    d1,6(a0)
                rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Left routine
;;
;;

move_jetman_left:
                move.w  #1,lr_pressed
                move.w  #1,facing               ; direction
                moveq   #0,d0
                move.w  jetman_y,d0             ; get y-values
                sub.w   #16,d0                  ; sub for top of jetman
                mulu    #448,d0                 ; mult by scanline width
                add.w   jetman_x,d0             ; add x-value on
                move.l  gamescr_addr,a0         ; get address of screen
                add.l   d0,a0                   ; address of top of jetman
                moveq   #20,d7                  ; 21 lines to check
.testloop:      tst.b   -2(a0)                  ; test
                bne.s   .noleft                 ;  each (& jump if hit!)
                lea     448(a0),a0              ;   left
                dbf     d7,.testloop            ;    pixel
                
                move.w  lanim_offset,d0         ; current animation
                add.w   #1,lanim_offset         ; next time....
                add.w   d0,d0
                add.w   d0,d0
                lea     left_anim_tab,a0
                move.l  (a0,d0.w),d0            ; address of grafix
                bpl.s   .nowrap
                move.w  #1,lanim_offset         ; reset pointers
                move.l  (a0),d0                 ; get 1st value

.nowrap:        move.l  d0,next_jetman_gfx        
                
                sub.w   #2,jetman_x             ; move left
                cmp.w   #-16,jetman_x           ; check for wrap
                bgt.s   .noleft                 ;
                move.w  #460,jetman_x           ; move to right edge

                tst.w   carry_rocket
                beq.s   .test_fuel
                move.l  rocket_pointer,a4
                move.w  #460,(a4)
.test_fuel:     tst.w   carry_fuel
                beq.s   .noleft
                move.w  #460,fuel_x

.noleft:        rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Right routine 
;;
;;

move_jetman_right:
                move.w  #1,lr_pressed
                move.w  #0,facing               ; direction
                moveq   #0,d0
                move.w  jetman_y,d0             ; y-position
                sub.w   #16,d0                  ; bottom of jetman
                mulu    #448,d0                 ; mult by scanline width
                add.w   jetman_x,d0             ; add on x-value
                move.l  gamescr_addr,a0         ; start of screen
                add.l   d0,a0                   ; offset into screen

                moveq   #20,d7                  ; 21 lines to check
.testloop:      tst.b   16(a0)                  ; test
                bne.s   .noright                ;  each (& jump if hit!)
                lea     448(a0),a0              ;   right
                dbf     d7,.testloop            ;    pixel

                move.w  ranim_offset,d0         ; current animation
                add.w   #1,ranim_offset         ; next time....
                add.w   d0,d0
                add.w   d0,d0
                lea     right_anim_tab,a0
                move.l  (a0,d0.w),d0            ; address of grafix
                bpl.s   .nowrap
                move.w  #1,ranim_offset         ; reset pointers
                move.l  (a0),d0                 ; get 1st value
.nowrap:
                move.l  d0,next_jetman_gfx                
                
                add.w   #2,jetman_x             ; move right
                cmp.w   #460,jetman_x           ; check screen edge
                blt.s   .noright
                move.w  #-16,jetman_x           ; move to left

                tst.w   carry_rocket
                beq.s   .test_fuel
                move.l  rocket_pointer,a4
                move.w  #-16,(a4)
.test_fuel:     tst.w   carry_fuel
                beq.s   .noright
                move.w  #-16,fuel_x

.noright:       rts

facing:         dc.w    0               ; 0-right, 1-left

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Up Routine
;;
;;

move_jetman_up: moveq   #0,d0
                move.w  jetman_y,d0             ; v-value
                sub.w   #18,d0                  ; top of jetman
                cmp.w   #20,d0                  ;
                bge     .up                     ; top of screen?
                bra.s   .noup
.up:            mulu    #448,d0                 ; mult by screen width
                add.w   jetman_x,d0             ; add on x-value
                move.l  gamescr_addr,a0
                add.l   d0,a0                   ; offset into screen
                moveq   #15,d7                  ; 16 pixels to check
.testloop:      tst.b   (a0)+                   ; test
                bne.s   .noup                   ;  above (& branch if hit!)
                dbf     d7,.testloop            ;   head
                sub.w   #2,jetman_y             ; shift 'im up!
.noup:          clr.w   lanim_offset            ; clear animation
                clr.w   ranim_offset            ; clear animation
                tst.w   facing                  ; which way?
                beq.s   .rightface
                lea     l_fly_anim_tab,a0       ; pointing left!
                bra.s   .newobj
.rightface:     lea     r_fly_anim_tab,a0       ; pointing right!
                
.newobj:        move.w  fly_anim_offset,d0     ; current animation
                add.w   #1,fly_anim_offset     ; next time....
                add.w   d0,d0
                add.w   d0,d0
                move.l  (a0,d0.w),d0            ; address of grafix
                bpl.s   .nowrap
                move.w  #1,fly_anim_offset      ; reset pointers
                move.l  (a0),d0                 ; get 1st value

.nowrap:        move.l  d0,next_jetman_gfx

                rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Gravity Routine
;;
;;

jetman_gravity: cmp.w   #214,jetman_y           ; bottom of screen?
                bge.s   .nodrop
                moveq   #0,d0
                move.w  jetman_y,d0             ; get y-value
                add.w   #6,d0                   ; get bottom of jetman
                mulu    #448,d0
                add.w   jetman_x,d0
                move.l  gamescr_addr,a0
                add.l   d0,a0                   ; offset into screen
                moveq   #15,d7                  ; 16 pixels to test
.testloop:      tst.b   (a0)+                   ; check 
                bne.s   .nodrop                 ;  below (& branch if hit!)
                dbf     d7,.testloop            ;   jetman
                add.w   #2,jetman_y             ; down!
                clr.w   lanim_offset            ; clear animation
                clr.w   ranim_offset            ; clear animation

                tst.w   facing                  ; which way?
                beq.s   .rightface
                lea     l_fly_anim_tab,a0       ; pointing left!
                bra.s   .newobj
.rightface:     lea     r_fly_anim_tab,a0       ; pointing right!
                
.newobj:        move.w  fly_anim_offset,d0     ; current animation
                add.w   #1,fly_anim_offset     ; next time....
                add.w   d0,d0
                add.w   d0,d0
                move.l  (a0,d0.w),d0            ; address of grafix
                bpl.s   .nowrap
                move.w  #1,fly_anim_offset      ; reset pointers
                move.l  (a0),d0                 ; get 1st value

.nowrap:        move.l  d0,next_jetman_gfx
                rts

.nodrop:        tst.w   lr_pressed              ; test left or right pushed
                bne.s   .rts                    ; if so, get outta here!
                tst.w   facing        
                beq.s   .faceright
                lea     left_anim_tab,a0        ; facing left
                bra.s   .allset
.faceright:     lea     right_anim_tab,a0       ; facing right
.allset:        move.l  (a0),next_jetman_gfx
                clr.w   fly_anim_offset         ; touched down!
.rts:           rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Object Initialization routine
;;
;;

setup_initial_objects:
                clr.w   rocket_built
                clr.w   mid_in_place

                move.l  rocketa_obj,a0
                move.w  #110,d0
                move.w  #87,d1
                move.w  #114,rocketax
                move.w  #87,rocketay
                bsr     setup_object_xy
                move.l  rocketb_obj,a0
                move.w  #240,d0
                move.w  #116,d1
                move.w  #246,rocketbx
                move.w  #116,rocketby
                bsr     setup_object_xy
                move.l  rocketc_obj,a0
                move.w  #290,d0
                move.w  #221,d1
                move.w  #290,rocketcx
                move.w  #221,rocketcy

                bsr     setup_object_xy
                move.l  gamescr_addr,a1
                lea     gamescr,a0              
                lea     game_pal,a2             
                moveq   #0,d5                   
                move.l  #448,d6                 
                move.l  #230,d7                 
                jsr     ConvertBMP              

init_jetman:
                move.w  #260,jetman_x           ; setup jetman
                move.w  #212,jetman_y
                clr.w   ranim_offset
                clr.w   lanim_offset
                clr.w   facing

                move.l  jmr1_addr,d0
                move.l  d0,next_jetman_gfx      ; initial jetman animation
                move.w  #260,jetman_x           ; setup jetman
                move.w  #212,jetman_y
                clr.w   ranim_offset
                clr.w   lanim_offset
                clr.w   facing

                rts

rocketax:       dc.w    110
rocketay:       dc.w    87
rockerafin:     dc.w    221-32
rocketbx:       dc.w    240
rocketby:       dc.w    116
rocketbfin:     dc.w    221-16
rocketcx:       dc.w    290
rocketcy:       dc.w    221
rocketcfin:     dc.w    221

setup_object_xy:        
                move.w  6(a0),d4                ; get y-pos
                and.w   #$c007,d4
                move.w  14(a0),d6               ; get x-pos
                and.w   #$f000,d6
                add.w   d1,d1
                lsl.w   #3,d1
                or.w    d1,d4
                or.w    d0,d6
                move.w  d4,6(a0)
                move.w  d6,14(a0)
                rts


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Wait for a VBL
;;

wait_frame:     move.w  d7,-(a7)
                move.w  ticks,d7
.wait:          cmp.w   ticks,d7
                beq.s   .wait
                move.w  (a7)+,d7
                rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Pallette Routine: fade to black
;;

scrmove_obj:    dc.l    0

fadepal_down:   move.w  #32,d6
.dofader:       lea     $f00400,a0      ; hardware pallette
                move.l  scrmove_obj,a3
                move.w  #255,d7         ; 254 colours
.palloop:       move.w  (a0),d0     
                move.w  d0,d1
                move.w  d0,d2
                and.w   #%1111100000000000,d0
                and.w   #%0000011111000000,d1
                and.w   #%0000000000111110,d2
.d0:            tst.w   d0
                beq.s   .d1
                sub.w   #%0000100000000000,d0
.d1:            tst.w   d1
                beq.s   .d2
                sub.w   #%0000000001000000,d1
.d2:            tst.w   d2
                beq.s   .merge
                sub.w   #%0000000000000010,d2
.merge:         or.w    d1,d0
                or.w    d2,d0
                move.w  d0,(a0)+
                dbf     d7,.palloop
                move.w  6(a3),d0
                move.w  d0,d1
                and.w   #$c007,d0
                and.w   #$3ff8,d1
                add.w   #10*8,d1
                or.w    d1,d0
                move.w  d0,6(a3)
                bsr     wait_frame
                dbf     d6,.dofader
                rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Pallette Routine: fade from black to required pallette
;;

fadepal_up:     move.w  #32,d6
                move.l  a1,a4
.dofader2:      lea     $f00400,a0      ; hardware pallette
                move.l  a4,a1
                move.l  scrmove_obj,a3
                move.w  #255,d7         ; 254 colours
.palloop2:      move.w  (a0),d0         ; word in pallette     
                move.w  (a1)+,d3        ; required word
                move.w  d0,d1
                move.w  d0,d2
                move.w  d3,d4
                move.w  d3,d5
                and.w   #%1111100000000000,d0
                and.w   #%0000011111000000,d1
                and.w   #%0000000000111110,d2
                and.w   #%1111100000000000,d3
                and.w   #%0000011111000000,d4
                and.w   #%0000000000111110,d5
.d0_1:          cmp.w   d0,d3
                beq.s   .d1_2
                add.w   #%0000100000000000,d0
.d1_2:          cmp.w   d1,d4
                beq.s   .d2_3
                add.w   #%0000000001000000,d1
.d2_3:          cmp.w   d2,d5
                beq.s   .merge_2
                add.w   #%0000000000000010,d2
.merge_2:       or.w    d1,d0
                or.w    d2,d0
                move.w  d0,(a0)+
                dbf     d7,.palloop2
                move.w  6(a3),d0
                move.w  d0,d1
                and.w   #$c007,d0
                and.w   #$3ff8,d1
                sub.w   #10*8,d1
                or.w    d1,d0
                move.w  d0,6(a3)
                bsr     wait_frame
                dbf     d6,.dofader2
                rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Vertical Blank Interupt (At least it happens every frame somewhere......)
;;

frcount:        dc.w    0
flash_rocket:   dc.w    0

VB_INTERUPT:    movem.l d0-d7/a0-a6,-(a7)

                jsr     copy_olist              ; use Blitter to update
                
                move.w  CONFIG,d0                ; Also is joystick register
                andi.w  #VIDTYPE,d0              ; 0 = PAL, 1 = NTSC
                beq     .palvals
                add.w   #1,frcount                
                cmp.w   #6,frcount
                bne.s   .palvals
                clr.w   frcount
                bra.s   .skipmusic

.palvals:       jsr     musicplayer+8
.skipmusic:     jsr     Random_Vbl
                
                tst.w   inside_game
                beq     .done_vbl
                
                jsr     print_lives
                
                bsr     plot_lasers             ; plot all active lasers
                jsr     animate_monsters                


                add.w   #%0000100001000010,$f00400+(200*2)

                tst.w   flash_rocket
                bne     .done_vbl

                sub.w   #1,fuel_flash_cnt
                bpl     .done_vbl
                move.w  #12,fuel_flash_cnt

                move.l  flash_colors,d1
                swap    d1
                move.l  d1,flash_colors

                lea     $f00400+(250*2),a0
                move.w  fuel_filled,d0
                beq.s   .whiteout
.flash:         move.w  d1,(a0)+
                sub.w   #1,d0
                bne.s   .flash
.whiteout:      cmp.l   #$f00400+(256*2),a0
                beq.s   .done_vbl
                move.w  #$ffff,(a0)+
                bra.s   .whiteout

.done_vbl:      move.w  #$101,INT1              ; Signal we're done
                move.w  #$0,INT2
                
                add.w   #1,ticks                ; new frame....

                movem.l (a7)+,d0-d7/a0-a6
                rte

fuel_flash_cnt: dc.w    12
flash_colors:   dc.w    $ffff,%1111111111000000

animate_monsters:
                sub.w   #1,animate_counter
                bmi.s   .do_anim
                rts
.do_anim:       move.w  #3,animate_counter
                move.w  monster_type,d5
                lsl.w   #3,d5
                lea     monster_animations,a3
                move.l  (a3,d5.w),a1            ; pointer to right anim ptr
                move.l  4(a3,d5.w),a2           ; pointer to left anim ptr

                move.l  (a1),a3                 ; pointer into r-table
                move.l  (a2),a4                 ; pointer into l-table

                add.l   #4,a3
                move.l  (a3),d5
                bpl.s   .ranimtabok
                move.l  4(a3),a3
.ranimtabok:    move.l  a3,(a1)                 ; update pointer d3=gfx long

                add.l   #4,a4
                move.l  (a4),d4
                bpl.s   .lanimtabok
                move.l  4(a4),a4
.lanimtabok:    move.l  a4,(a2)                 ; update pointer d4-gfx long

                rts

animate_counter:        dc.w    3

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Create the initial object list.
;;

bitmapobject    equ 0            ; object types
scaledobject    equ 1
gpuobject       equ 2
braobject       equ 3             
stopobject      equ 4

y_less          equ 0            ; branch object types
y_more          equ 1
always          equ 2


MakeObjectList: lea     ob_list1,a0             ; start of OL
                jsr     build_header            ; build BRA header
                                       
                move.l  a0,screen_obj           ; store build address
                lea     $100000,a1
                move.l  #bitmapobject,d0        ; type
                move.l  #230,d1                 ; height
                move.l  #448,d2                 ; bytes to next line
                move.l  #3,d3                   ; 8-Bit
                move.l  #0,d4                   ; non transparent
                move.l  #448,d5                 ; bytes/pixels*width / line
                jsr     CreateObject
                
                move.l  a0,rocketa_obj          ; store build address
                move.l  rocket1_addr,a1
                move.l  #bitmapobject,d0        ; type
                move.l  #16,d1                  ; height
                move.l  #16,d2                  ; bytes to next line
                move.l  #3,d3   
                move.l  #1,d4                   ; non transparent
                move.l  #16,d5                  ; bytes/pixels*width / line
                jsr     CreateObject
                
                move.l  a0,rocketb_obj          ; store build address
                move.l  rocket1_addr,a1
                lea     16*16(a1),a1
                move.l  #bitmapobject,d0        ; type
                move.l  #16,d1                  ; height
                move.l  #16,d2                  ; bytes to next line
                move.l  #3,d3   
                move.l  #1,d4                   ; non transparent
                move.l  #16,d5                  ; bytes/pixels*width / line
                jsr     CreateObject
                
                move.l  a0,rocketc_obj          ; store build address
                move.l  rocket1_addr,a1
                lea     32*16(a1),a1
                move.l  #bitmapobject,d0        ; type
                move.l  #16,d1                  ; height
                move.l  #16,d2                  ; bytes to next line
                move.l  #3,d3   
                move.l  #1,d4                   ; non transparent
                move.l  #16,d5                  ; bytes/pixels*width / line
                jsr     CreateObject
                
                move.l  a0,jetman_obj           ; store build address
                move.l  jmr1_addr,a1
                move.l  #bitmapobject,d0        ; type
                move.l  #24,d1                  ; height
                move.l  #16*2,d2                ; bytes to next line
                move.l  #4,d3   
                move.l  #1,d4                   ; non transparent
                move.l  #16*2,d5                ; bytes/pixels*width / line
                jsr     CreateObject
                
                move.l  a0,fuel_obj             ; store build address
                move.l  fuel_addr,a1
                move.l  #bitmapobject,d0        ; type
                move.l  #11,d1                  ; height
                move.l  #16*2,d2                ; bytes to next line
                move.l  #4,d3   
                move.l  #1,d4                   ; non transparent
                move.l  #16*2,d5                ; bytes/pixels*width / line
                jsr     CreateObject
                
                move.l  a0,monster1_obj         ; store build address
                move.l  roid1_addr,a1
                move.l  #bitmapobject,d0        ; type
                move.l  #14,d1                  ; height
                move.l  #16*2,d2                ; bytes to next line
                move.l  #4,d3   
                move.l  #1,d4                   ; non transparent
                move.l  #16*2,d5                ; bytes/pixels*width / line
                jsr     CreateObject
                
                move.l  a0,monster2_obj         ; store build address
                move.l  roid1_addr,a1
                move.l  #bitmapobject,d0        ; type
                move.l  #14,d1                  ; height
                move.l  #16*2,d2                ; bytes to next line
                move.l  #4,d3   
                move.l  #1,d4                   ; non transparent
                move.l  #16*2,d5                ; bytes/pixels*width / line
                jsr     CreateObject
                
                move.l  a0,monster3_obj         ; store build address
                move.l  roid1_addr,a1
                move.l  #bitmapobject,d0        ; type
                move.l  #14,d1                  ; height
                move.l  #16*2,d2                ; bytes to next line
                move.l  #4,d3   
                move.l  #1,d4                   ; non transparent
                move.l  #16*2,d5                ; bytes/pixels*width / line
                jsr     CreateObject
                
                move.l  a0,monster4_obj         ; store build address
                move.l  roid1_addr,a1
                move.l  #bitmapobject,d0        ; type
                move.l  #14,d1                  ; height
                move.l  #16*2,d2                ; bytes to next line
                move.l  #4,d3   
                move.l  #1,d4                   ; non transparent
                move.l  #16*2,d5                ; bytes/pixels*width / line
                jsr     CreateObject
                
                move.l  a0,monster5_obj         ; store build address
                move.l  roid1_addr,a1
                move.l  #bitmapobject,d0        ; type
                move.l  #14,d1                  ; height
                move.l  #16*2,d2                ; bytes to next line
                move.l  #4,d3   
                move.l  #1,d4                   ; non transparent
                move.l  #16*2,d5                ; bytes/pixels*width / line
                jsr     CreateObject
                
                move.l  a0,monster6_obj         ; store build address
                move.l  roid1_addr,a1
                move.l  #bitmapobject,d0        ; type
                move.l  #14,d1                  ; height
                move.l  #16*2,d2                ; bytes to next line
                move.l  #4,d3   
                move.l  #1,d4                   ; non transparent
                move.l  #16*2,d5                ; bytes/pixels*width / line
                jsr     CreateObject
                
                moveq.l #stopobject,d0          ; STOP Object
                jsr     CreateObject
                rts

title_addr:     dc.l    0
gamescr_addr:   dc.l    0
screen_addr:    dc.l    0
screen_obj:     dc.l    0
rocket1_addr:   dc.l    0
rocketa_obj:    dc.l    0
rocketb_obj:    dc.l    0
rocketc_obj:    dc.l    0
fuel_obj:       dc.l    0
fuel_addr:      dc.l    0
monster1_obj:   dc.l    0
monster2_obj:   dc.l    0
monster3_obj:   dc.l    0
monster4_obj:   dc.l    0
monster5_obj:   dc.l    0
monster6_obj:   dc.l    0

monster_animations:     dc.l    roid_roffset,roid_loffset

roid1_addr:     dc.l    0
roid2_addr:     dc.l    0
roidl1_addr:    dc.l    0
roidl2_addr:    dc.l    0
roid_roffset:   dc.l    roid_ranim_tab
roid_loffset:   dc.l    roid_lanim_tab
roid_ranim_tab: dc.l    0,0,-1,roid_ranim_tab
roid_lanim_tab: dc.l    0,0,-1,roid_lanim_tab

ranim_offset:   dc.w    0
lanim_offset:   dc.w    0
left_anim_tab:  dc.l    0,0,0,0,0,0,0,0,-1
right_anim_tab: dc.l    0,0,0,0,0,0,0,0,-1

fly_anim_offset:dc.w    0
l_fly_anim_tab: dc.l    0,0,0,0,0,0,0,0,-1
r_fly_anim_tab: dc.l    0,0,0,0,0,0,0,0,-1

jmr1_addr:      dc.l    0
jml1_addr:      dc.l    0
jmr2_addr:      dc.l    0
jml2_addr:      dc.l    0
jmr3_addr:      dc.l    0
jml3_addr:      dc.l    0

jfr1_addr:      dc.l    0
jfr2_addr:      dc.l    0
jfr3_addr:      dc.l    0
jfl1_addr:      dc.l    0
jfl2_addr:      dc.l    0
jfl3_addr:      dc.l    0

jetman_obj:     dc.l    0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Call above to make the BMPs
;;

ConvertBMPtoJAG:
                lea     $100000,a1
                move.l  a1,title_addr
                lea     title,a0                
                lea     title_pal,a2            
                moveq   #0,d5                   
                move.l  #448,d6                 
                move.l  #230,d7                 
                jsr     ConvertBMP              
                
                move.l  d0,a1
                move.l  a1,gamescr_addr
                lea     gamescr,a0              
                lea     game_pal,a2             
                moveq   #0,d5                   
                move.l  #448,d6                 
                move.l  #230,d7                 
                jsr     ConvertBMP              

                move.l  d0,a1
                move.l  a1,rocket1_addr
                lea     rocket1,a0              
                lea     tmppal,a2             
                moveq   #0,d5                   
                move.l  #16,d6                 
                move.l  #48,d7                 
                jsr     ConvertBMP              
                
                movem.l tmppal+(250*2),d1-d3
                movem.l d1-d3,game_pal+(250*2)

                move.l  d0,a1
                move.l  a1,fuel_addr
                lea     fuel,a0              
                lea     tmppal,a2             
                moveq   #1,d5                   
                move.l  #16,d6                 
                move.l  #11,d7                 
                jsr     ConvertBMP              

                move.l  d0,a1
                move.l  a1,jmr1_addr
                lea     jetman_r1,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #24,d7
                bsr     ConvertBMP
                move.l  d0,a1
                move.l  a1,jmr2_addr
                lea     jetman_r2,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #24,d7
                bsr     ConvertBMP
                move.l  d0,a1
                move.l  a1,jmr3_addr
                lea     jetman_r3,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #24,d7
                bsr     ConvertBMP
                
                move.l  d0,a1
                move.l  a1,jml1_addr
                lea     jetman_l1,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #24,d7
                bsr     ConvertBMP
                move.l  d0,a1
                move.l  a1,jml2_addr
                lea     jetman_l2,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #24,d7
                bsr     ConvertBMP
                move.l  d0,a1
                move.l  a1,jml3_addr
                lea     jetman_l3,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #24,d7
                bsr     ConvertBMP
                
                move.l  d0,a1
                move.l  a1,jfr1_addr
                lea     jetfly_r1,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #24,d7
                bsr     ConvertBMP
                move.l  d0,a1
                move.l  a1,jfr2_addr
                lea     jetfly_r2,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #24,d7
                bsr     ConvertBMP
                move.l  d0,a1
                move.l  a1,jfr3_addr
                lea     jetfly_r3,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #24,d7
                bsr     ConvertBMP
                
                move.l  d0,a1
                move.l  a1,jfl1_addr
                lea     jetfly_l1,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #24,d7
                bsr     ConvertBMP
                move.l  d0,a1
                move.l  a1,jfl2_addr
                lea     jetfly_l2,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #24,d7
                bsr     ConvertBMP
                move.l  d0,a1
                move.l  a1,jfl3_addr
                lea     jetfly_l3,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #24,d7
                bsr     ConvertBMP
                
                move.l  d0,a1
                move.l  a1,roid1_addr
                lea     roid1,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #14,d7
                bsr     ConvertBMP
                move.l  d0,a1
                move.l  a1,roid2_addr
                lea     roid2,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #14,d7
                bsr     ConvertBMP

                move.l  d0,a1
                move.l  a1,roidl1_addr
                lea     roidl1,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #14,d7
                bsr     ConvertBMP
                move.l  d0,a1
                move.l  a1,roidl2_addr
                lea     roidl2,a0
                lea     tmppal,a2
                moveq   #1,d5                   ; 16 bit
                move.l  #16,d6
                move.l  #14,d7
                bsr     ConvertBMP

                move.l  d0,last_gfx

                move.l  roid1_addr,d0
                move.l  roid2_addr,d1
                lea     roid_ranim_tab,a0
                move.l  d0,(a0)+
                move.l  d1,(a0)
                move.l  roidl1_addr,d0
                move.l  roidl2_addr,d1
                lea     roid_lanim_tab,a0
                move.l  d1,(a0)+
                move.l  d0,(a0)
                
                move.l  jmr1_addr,d0
                move.l  jmr2_addr,d1
                move.l  jmr3_addr,d2
                move.l  jml1_addr,d3
                move.l  jml2_addr,d4
                move.l  jml3_addr,d5
                lea     left_anim_tab,a0
                move.l  d3,(a0)+
                move.l  d3,(a0)+
                move.l  d4,(a0)+
                move.l  d4,(a0)+
                move.l  d5,(a0)+
                move.l  d5,(a0)+
                move.l  d4,(a0)+
                move.l  d4,(a0)
                lea     right_anim_tab,a0
                move.l  d0,(a0)+
                move.l  d0,(a0)+
                move.l  d1,(a0)+
                move.l  d1,(a0)+
                move.l  d2,(a0)+
                move.l  d2,(a0)+
                move.l  d1,(a0)+
                move.l  d1,(a0)
                
                move.l  jfr1_addr,d0
                move.l  jfr2_addr,d1
                move.l  jfr3_addr,d2
                move.l  jfl1_addr,d3
                move.l  jfl2_addr,d4
                move.l  jfl3_addr,d5
                lea     l_fly_anim_tab,a0
                move.l  d3,(a0)+
                move.l  d3,(a0)+
                move.l  d4,(a0)+
                move.l  d4,(a0)+
                move.l  d5,(a0)+
                move.l  d5,(a0)+
                move.l  d4,(a0)+
                move.l  d4,(a0)
                lea     r_fly_anim_tab,a0
                move.l  d0,(a0)+
                move.l  d0,(a0)+
                move.l  d1,(a0)+
                move.l  d1,(a0)+
                move.l  d2,(a0)+
                move.l  d2,(a0)+
                move.l  d1,(a0)+
                move.l  d1,(a0)
                
                rts

last_gfx:       dc.l    0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Clear ram routine
;;

clear_memory:   move.l a1,d0
                sub.l a0,d0
                asr.l #2,d0
.wipe:          clr.l (a0)+
                subq.l #1,d0
                bne.s .wipe
                rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Copy ram routine
;;

copy_ram:       move.w (a0)+,(a2)+
                cmp.l a0,a1
                bne.s copy_ram
                rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Procedure: InitVideo (same as in vidinit.s)
;;            Build values for hdb, hde, vdb, and vde and store them.
;;

InitVideo:
                movem.l d0-d6,-(sp)
                move.w  CONFIG,d0                ; Also is joystick register
                andi.w  #VIDTYPE,d0              ; 0 = PAL, 1 = NTSC
                beq     palvals
                move.w  #NTSC_HMID,d2
                move.w  #NTSC_WIDTH,d0
                move.w  #NTSC_VMID,d6
                move.w  #NTSC_HEIGHT,d4
                bra     calc_vals
palvals:
                move.w  #PAL_HMID,d2
                move.w  #PAL_WIDTH,d0
                move.w  #PAL_VMID,d6
                move.w  #PAL_HEIGHT,d4
calc_vals:
                move.w  d0,width
                move.w  d4,height
                move.w  d0,d1
                asr     #1,d1                   ; Width/2
                sub.w   d1,d2                   ; Mid - Width/2
                add.w   #4,d2                   ; (Mid - Width/2)+4
                sub.w   #1,d1                   ; Width/2 - 1
                ori.w   #$400,d1                ; (Width/2 - 1)|$400
                move.w  d1,a_hde
                move.w  d1,HDE
                move.w  d2,a_hdb
                move.w  d2,HDB1
                move.w  d2,HDB2
                move.w  d6,d5
                sub.w   d4,d5
                add.w   #16,d5
                move.w  d5,a_vdb
                add.w   d4,d6
                move.w  d6,a_vde
                move.w  a_vdb,VDB
                move.w  #$FFFF,VDE
                move.l  #0,BORD1                ; Black border
                move.w  #0,BG                   ; Init line buffer to black
                movem.l (sp)+,d0-d6
                rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Copy Object List Routine
;;

copy_olist:     move.l  #0,d7                  ; Gordon/Atari mess
                move.l  #ob_list1,A2_BASE
                move.l  #ob_list,A1_BASE
                move.l  d7,A1_PIXEL
                move.l  d7,A2_PIXEL
                move.w  #1,d0
                swap    d0
                move.w  #(objs*32)+64,d0
                move.l  d0,B_COUNT              ;$00010001 = 8 Bytes
                move.l  #PIXEL32|XADDPHR,d6
                move.l  d6,A1_FLAGS
                move.l  d6,A2_FLAGS
                move.l  #SRCEN|LFU_S,d7
                move.l  d7,B_CMD
                move.l  #0,A1_CLIP
                rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Build a header that'll STOP if VPOS not on screen.
;;

build_header:   move.l  a0,d0               ; address if bra not taken
                sub.l   #ob_list1,d0        ;
                add.l   #ob_list,d0         ;
                add.l   #32,d0              ; next BRA object
                lsr.l   #3,d0               ;
                lsl.l   #8,d0               ;
                clr.l   (a0)                ;
                move.l  #$00008003,4(a0)    ; bra if yp<0
                or.l    d0,2(a0)            ; link to next BRA
                lea     8(a0),a0            ;
                bsr     stopob              ; write stop object (before top)
                lea     16(a0),a0           ; next object here

                move.l  a0,d0               ; calc address of 1st actual
                sub.l   #ob_list1,d0        ; object (after the next STOP)
                add.l   #ob_list,d0         ;
                add.l   #32,d0              ;
                lsr.l   #3,d0               ;
                lsl.l   #8,d0               ;
                clr.l   (a0)                ;
                move.l  #$00004003,4(a0)    ; bra is yp>value
                or.l    d0,2(a0)            ; link to first object!
                move.l  #492,d0             ; 460/2 = 230 scanlines!
                lsl.l   #3,d0               ;
                or.l    d0,4(a0)            ;
                lea     8(a0),a0            ; next object
                bsr     stopob              ; is a STOP
                lea     16(a0),a0           ; address for next REAL object

                rts

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Simple Object List Routines for a simple life.
;;
;; call with *d0=object type
;;           *d1=height
;;           *d2=data width
;;           *d3=colour depth
;;           *d4=transparent
;;           *d5=image width
;;           *a0=address of object (phrase alligned)
;;           *a1=address for GFX   (phrase alligned)
;;
;; exit with object built
;;           link=object address+32
;;           scaled objects will *NOT* be 1:1 in x/y
;;           x/y positions will be -500,2 (off screen)
;;

CreateObject:   lsl.w   #2,d0                   ; object type
                move.l  jmp_tab(pc,d0),a6       ; get routine address
                jmp (a6)                        ; call it!

jmp_tab:        .dc.l bitmp                     ; jump table
                .dc.l scaled
                .dc.l gpuob
                .dc.l braob
                .dc.l stopob

bitmp:          clr.l   (a0)                    ; template
                clr.l   4(a0)                   ;
                clr.l   8(a0)                   ;
                move.l  #$00008000,12(a0)       ;
                move.l  a0,d0                   ; link address
                sub.l   #ob_list1,d0            ;
                add.l   #ob_list,d0             ;
                add.l   #32,d0                  ;
                and.b   #%11111000,d0           ;
                lsl.l   #5,d0                   ;
                or.l    d0,2(a0)                ;
                move.l  a1,d0                   ; gfx address
                lsl.l   #8,d0                   ;
                or.l    d0,(a0)                 ;
                move.l  d1,d0                   ; height
                swap    d0                      ;
                lsr.l   #2,d0                   ;
                or.l    d0,4(a0)                ;
                ror.w   #1,d4                   ; transparency
                or.w    d4,10(a0)               ;
                lsl.w   #8,d3                   ; depth (colour depth)
                lsl.w   #4,d3                   ;
                or.w    d3,14(a0)               ;
                lsr.w   #3,d2                   ; data width
                swap    d2                      ;
                lsl.l   #2,d2                   ;
                or.l    d2,12(a0)               ;
                lsr.w   #3,d5                   ; image width
                swap    d5                      ;
                lsr.l   #4,d5                   ;
                or.l    d5,10(a0)               ;
                move.w  #-500,d0                ; x-pos
                and.w   #$fff,d0
                or.w    d0,14(a0)
                or.w    #2*8,6(a0)              ; y-pos
                lea     32(a0),a0               ; BITMAP object!
                rts

scaled:         bsr     bitmp                   ; same as bitmap
                move.l  #$0,-16(a0)             ; clear it out
                move.l  #%00000000000000000100000001000000,-12(a0)
                or.l    #$1,-28(a0)             ; SCALED object!
                rts

gpuob:          move.l  #0,(a0)+
                move.l  #$3ffa,(a0)+            ; GPU object!
                rts

braob:          add     d2,d2                   ; mult y-pos
                bsr     branchobject            ; make the object
make8into32:    move.l  #braobject,d0           ;
                move.l  #always,d1              ; even it out for 32-byte
                move.l  #$7ff,d2                ; positions by following with
                lea     24(a0),a1               ; a BRA+16 object
                bsr     branchobject            ;
                lea     16(a0),a0               ;
                rts

branchobject:   clr.l   (a0)
                move.l  #3,4(a0)
                add.w   d1,d1
                move.w  branchtypes(pc,d1.w),d1
                or.w    d1,6(a0)                ; branch TYPE!
                move.l  a1,d0
                sub.l   #ob_list1,d0
                add.l   #ob_list,d0
                and.l   #$fffffff8,d0
                lsl.l   #5,d0                   ; Link if branch **TAKEN!**
                move.l  d0,2(a0)                ; (< & > swapped!)
                lsl.w   #3,d2                   ; scanline to branch on
                or.w    d2,6(a0)                ; is VC/2
                lea     8(a0),a0                ; next object
                rts

branchtypes:    dc.w    $4000,$8000,$0000       ; (< & > inverted for ease!)

stopob:         move.l  #0,(a0)+
                move.l  #4,(a0)+                ; STOP object!
                rts



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Assumes 8bpp and RGB - Will not handle RLE8 Packed Files!!!
;;
;; a0 - address of Windows bitmap
;; a1 - address to place Jaguar bitmap at
;; a2 - address to place 16-bit Jaguar pallette
;; d5 - 0=8bit colour JAG dest. bitmap,  1=16bit colour JAG dest. bitmap
;; d6 - width (in BYTES)
;; d7 - height (in SCANLINES)
;;
;; returns GFX end in d0

ConvertBMP:     move.l  a2,-(a7)
                move.l  a0,a6
                lea     53(a6),a3       ; Start of xBGR table
                move.l  #255,d0         ; 256 colours
.clut:          moveq.l #0,d1           ;
                moveq.l #0,d2           ; make
                moveq.l #0,d3           ;  em
                moveq.l #0,d4           ;   empty
                move.b  (a3)+,d1        ; unused
                move.b  (a3)+,d3        ; (B)lue
                move.b  (a3)+,d1        ; (G)reen
                move.b  (a3)+,d2        ; (R)ed
                lsr.w   #3,d2           ; div 8 for 5-bit colour
                ror.w   #5,d2           ; shift to hi-bits
                or.w    d2,d4           ; mask RED!
                lsr.w   #3,d3           ; div 8 for 5-bit colour
                rol.w   #6,d3           ; shift to mid-bits
                or.w    d3,d4           ; mask BLUE!
                lsr.w   #2,d1           ; div 4 for 6-bit colour
                or.w    d1,d4           ; mask GREEN!
                move.w  d4,(a2)+        ; Store 16-BIT RBG Jag Colour!
                dbra    d0,.clut
                move.l  (a7)+,a2

                tst.l   d5              ; check for colour depth
                beq     eightbit

sixteenbit:     lea     1024+54(a6),a0  ;
                move.w  d6,d0           ;
                muls    d7,d0           ;
                add.l   d0,a1           ;
                add.l   d0,a1           ;
                move.l  a1,-(a7)
                sub.l   d6,a1           ;
                sub.l   d6,a1           ; addr of last line

                move.l  d7,d0           ; copy the bitmap
                subq.l  #1,d0
.lines:         move.l  d6,d1           ; a line at a time
                subq.l  #1,d1
.acc:           move.b  (a0)+,d3        ; get colour value
                and.w   #$00FF,D3       ; and only that value
                add.w   d3,d3           ; mult by 2
                move.w  (a2,d3.w),(a1)+ ; store RBG Jag value from pallette
                dbra    d1,.acc         ; loop for pixels
                sub.l   d6,a1           ;
                sub.l   d6,a1           ; go back
                sub.l   d6,a1           ;  2 lines
                sub.l   d6,a1           ;
                dbra    d0,.lines       ; loop for lines

                move.l  (a7)+,d0        ; get end of gfx address
                rts

eightbit:       lea     1024+54(a6),a0  ;
                move.l  d7,d5
                subq    #1,d5
                muls    d6,d5
                add.l   d5,a1
                add.l   d6,a1
                move.l  a1,-(a7)
                sub.l   d6,a1

.lines:         move.l  d6,d1           ; a line at a time
.acc:           move.b  (a0)+,(a1)+
                subq    #1,d1
                bne.s   .acc            ; loop for pixels
                sub.l   d6,a1           ;
                sub.l   d6,a1           ; go back
                subq    #1,d7
                bne.s   .lines

                move.l  (a7)+,d0
                rts

Init_music:     jsr     musicplayer
                moveq   #128,d0
                bsr     musicplayer+$14              ; mid range volume
                moveq   #128,d0
                bsr     musicplayer+$18
                lea     module,a0
                jsr     musicplayer+4
                rts

;text @ a0 plotted @ a1

print_lives:    lea     lives_txt,a0
                move.l  gamescr_addr,a1
                add.l   #(448*5)+100,a1

Print8:         moveq   #0,d0
                move.b  (a0)+,d0
                bpl.s   .notend
                rts
.notend:        sub.b   #' ',d0
                lsl.w   #6,d0                   ; x64
                lea     small_font,a3
                add.l   d0,a3
                move.l  a1,a4
                bsr     .plot_char
                lea     8(a1),a1
                bra     Print8
.plot_char:     moveq   #7,d7
.loop_4:        movem.l (a3)+,d0-d1
                movem.l d0-d1,(a4)
                lea     448(a4),a4
                dbf     d7,.loop_4
                rts

lives_txt:      dc.b    "LIVES: "
jetmen:         dc.b    "3",-1
                even
small_font:     incbin "8bitfont.jag"
                .long


                include "random.s"
                include "play.s"

                .phrase

                even
title:          incbin "titlebug.bmp"
                even
gamescr:        incbin "gamescr.bmp"
                even
module:         incbin "salute.mod"
                even

jetman_r1:      incbin "jmr1.bmp"               ; jetman walking right
jetman_r2:      incbin "jmr2.bmp"
jetman_r3:      incbin "jmr3.bmp"
jetman_l1:      incbin "jml1.bmp"               ; jetman walking left
jetman_l2:      incbin "jml2.bmp"
jetman_l3:      incbin "jml3.bmp"
jetfly_r1:      incbin "jflyr1.bmp"             ; jetman flying right
jetfly_r2:      incbin "jflyr2.bmp"
jetfly_r3:      incbin "jflyr3.bmp"
jetfly_l1:      incbin "jflyl1.bmp"             ; jetman flying left
jetfly_l2:      incbin "jflyl3.bmp"
jetfly_l3:      incbin "jflyl3.bmp"

roid1:          incbin "roid1.bmp"              ; asteroid
roid2:          incbin "roid2.bmp"
roidl1:         incbin "roidl1.bmp"
roidl2:         incbin "roidl2.bmp"

fuel:           incbin "fuel.bmp"               ; case of fuel

rocket1:        incbin "rocket.bmp"             ; The "Ultimate One"

                .bss
                .dphrase

ticks:          .ds.l   1                       ; Incrementing # of ticks
a_hdb:          .ds.w   1
a_hde:          .ds.w   1
a_vdb:          .ds.w   1
a_vde:          .ds.w   1
width:          .ds.w   1
height:         .ds.w   1
title_pal:      .ds.w   256
game_pal:       .ds.w   256
tmppal:         .ds.w   256
                .end




